@@ -1,5 +1,3 @@
-from __future__ import absolute_import, print_function
-
 import tweepy
 
 # == OAuth Authentication ==
@@ -19,14 +17,13 @@ access_token=""
 access_token_secret=""
 
 auth = tweepy.OAuthHandler(consumer_key, consumer_secret)
-auth.secure = True
 auth.set_access_token(access_token, access_token_secret)
 
 api = tweepy.API(auth)
 
 # If the authentication was successful, you should
 # see the name of the account print out
-print(api.me().name)
+print(api.verify_credentials().name)
 
 # If the application settings are set for "Read and Write" then
 # this line should tweet out the message to your account's
