@@ -1,18 +1,17 @@
-from __future__ import absolute_import
-
 import random
 import unittest
 
-from six.moves import input
-
-from .config import *
+from tests.config import *
 from tweepy import API, OAuthHandler
 
 
 class TweepyAuthTests(unittest.TestCase):
 
     def testoauth(self):
-        auth = OAuthHandler(oauth_consumer_key, oauth_consumer_secret)
+        if not consumer_key or not consumer_secret:
+            self.skipTest("Missing consumer key and/or secret")
+
+        auth = OAuthHandler(consumer_key, consumer_secret)
 
         # test getting access token
         auth_url = auth.get_authorization_url()
@@ -24,11 +23,14 @@ class TweepyAuthTests(unittest.TestCase):
 
         # build api object test using oauth
         api = API(auth)
-        s = api.update_status('test %i' % random.randint(0, 1000))
+        s = api.update_status(f'test {random.randint(0, 1000)}')
         api.destroy_status(s.id)
 
     def testaccesstype(self):
-        auth = OAuthHandler(oauth_consumer_key, oauth_consumer_secret)
+        if not consumer_key or not consumer_secret:
+            self.skipTest("Missing consumer key and/or secret")
+
+        auth = OAuthHandler(consumer_key, consumer_secret)
         auth_url = auth.get_authorization_url(access_type='read')
         print('Please open: ' + auth_url)
         answer = input('Did Twitter only request read permissions? (y/n) ')
