@@ -19,6 +19,7 @@ from tweepy.errors import (
 from tweepy.media import Media
 from tweepy.place import Place
 from tweepy.poll import Poll
+from tweepy.space import Space
 from tweepy.tweet import Tweet
 from tweepy.user import User
 
@@ -92,12 +93,14 @@ class Client:
             auth = auth.apply_auth()
         else:
             headers["Authorization"] = f"Bearer {self.bearer_token}"
+
         log.debug(
             f"Making API request: {method} {host + route}\n"
             f"Parameters: {params}\n"
             f"Headers: {headers}\n"
             f"Body: {json}"
         )
+
         with self.session.request(
             method, host + route, params=params, json=json, headers=headers,
             auth=auth
@@ -108,6 +111,7 @@ class Client:
                 f"Headers: {response.headers}\n"
                 f"Content: {response.content}"
             )
+
             if response.status_code == 400:
                 raise BadRequest(response)
             if response.status_code == 401:
@@ -132,26 +136,27 @@ class Client:
                 raise TwitterServerError(response)
             if not 200 <= response.status_code < 300:
                 raise HTTPException(response)
+
             return response
 
     def _make_request(self, method, route, params={}, endpoint_parameters=None,
                       json=None, data_type=None, user_auth=False):
         request_params = {}
         for param_name, param_value in params.items():
-            if param_name in endpoint_parameters:
-                if isinstance(param_value, list):
-                    request_params[param_name] = ','.join(map(str, param_value))
-                elif param_name in ("start_time", "end_time") and isinstance(param_value, datetime.datetime):
-                    if param_value.tzinfo is not None:
-                        param_value = param_value.astimezone(datetime.timezone.utc)
-                    request_params[param_name] = param_value.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
-                    # TODO: Constant datetime format string?
-                else:
-                    request_params[param_name] = param_value
-            elif param_name.replace('_', '.') in endpoint_parameters:
-                # Use := when support for Python 3.7 is dropped
-                request_params[param_name.replace('_', '.')] = ','.join(param_value)
+            if param_name.replace('_', '.') in endpoint_parameters:
+                param_name = param_name.replace('_', '.')
+
+            if isinstance(param_value, list):
+                request_params[param_name] = ','.join(map(str, param_value))
+            elif isinstance(param_value, datetime.datetime):
+                if param_value.tzinfo is not None:
+                    param_value = param_value.astimezone(datetime.timezone.utc)
+                request_params[param_name] = param_value.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
+                # TODO: Constant datetime format string?
             else:
+                request_params[param_name] = param_value
+
+            if param_name not in endpoint_parameters:
                 log.warn(f"Unexpected parameter: {param_name}")
 
         response = self.request(method, route, params=request_params,
@@ -1359,6 +1364,48 @@ class Client:
 
         return self._make_request("DELETE", route, user_auth=True)
 
+    def get_muted(self, **params):
+        """get_muted(*, expansions, max_results, pagination_token, \
+                     tweet_fields, user_fields)
+
+        Returns a list of users who are muted by the authenticating user.
+
+        Parameters
+        ----------
+        expansions : Union[List[str], str]
+            :ref:`expansions_parameter`
+        max_results : int
+            The maximum number of results to be returned per page. This can be
+            a number between 1 and 1000. By default, each page will return 100
+            results.
+        pagination_token : str
+            Used to request the next page of results if all results weren't
+            returned with the latest request, or to go back to the previous
+            page of results.
+        tweet_fields : Union[List[str], str]
+            :ref:`tweet_fields_parameter`
+        user_fields : Union[List[str], str]
+            :ref:`user_fields_parameter`
+
+        Returns
+        -------
+        Union[dict, requests.Response, Response]
+
+        References
+        ----------
+        https://developer.twitter.com/en/docs/twitter-api/users/mutes/api-reference/get-users-muting
+        """
+        id = self.access_token.partition('-')[0]
+        route = f"/2/users/{id}/muting"
+
+        return self._make_request(
+            "GET", route, params=params,
+            endpoint_parameters=(
+                "expansions", "max_results", "pagination_token",
+                "tweet.fields", "user.fields"
+            ), data_type=User, user_auth=True
+        )
+
     def mute(self, target_user_id):
         """Allows an authenticated user ID to mute the target user.
 
@@ -1499,3 +1546,226 @@ class Client:
                 "ids", "usernames", "expansions", "tweet.fields", "user.fields"
             ), data_type=User, user_auth=user_auth
         )
+
+    # Search Spaces
+
+    def search_spaces(self, query, state, **params):
+        """search_spaces(query, state, *, expansions, max_results, \
+                         space_fields, user_fields)
+
+        Return live or scheduled Spaces matching your specified search terms
+
+        Parameters
+        ----------
+        query : str
+            Your search term. This can be any text (including mentions and
+            Hashtags) present in the title of the Space.
+        state : str
+            Determines the type of results to return. Use ``live`` to return
+            live Spaces or ``scheduled`` to return upcoming Spaces.
+        expansions : Union[List[str], str]
+            :ref:`expansions_parameter`
+        max_results : int
+            The maximum number of results to return in this request. Specify a
+            value between 1 and 100.
+        space_fields : Union[List[str], str]
+            :ref:`space_fields_parameter`
+        user_fields : Union[List[str], str]
+            :ref:`user_fields_parameter`
+
+        Returns
+        -------
+        Union[dict, requests.Response, Response]
+
+        References
+        ----------
+        https://developer.twitter.com/en/docs/twitter-api/spaces/search/api-reference/get-spaces-search
+        """
+        params["query"] = query
+        params["state"] = state
+        return self._make_request(
+            "GET", "/2/spaces/search", params=params,
+            endpoint_parameters=(
+                "query", "state", "expansions", "max_results", "space.fields",
+                "user.fields"
+            ), data_type=Space
+        )
+
+    # Spaces lookup
+
+    def get_spaces(self, *, ids=None, user_ids=None, **params):
+        """get_spaces(*, ids, user_ids, expansions, space_fields, user_fields)
+
+        Returns details about multiple live or scheduled Spaces (created by the
+        specified user IDs if specified). Up to 100 comma-separated Space or
+        user IDs can be looked up using this endpoint.
+
+        Parameters
+        ----------
+        ids : Union[List[str], str]
+            A comma separated list of Spaces (up to 100).
+        user_ids : Union[List[int, str], str]
+            A comma separated list of user IDs (up to 100).
+        expansions : Union[List[str], str]
+            :ref:`expansions_parameter`
+        space_fields : Union[List[str], str]
+            :ref:`space_fields_parameter`
+        user_fields : Union[List[str], str]
+            :ref:`user_fields_parameter`
+
+        Raises
+        ------
+        TypeError
+            If IDs and user IDs are not passed or both are passed
+
+        Returns
+        -------
+        Union[dict, requests.Response, Response]
+
+        References
+        ----------
+        https://developer.twitter.com/en/docs/twitter-api/spaces/lookup/api-reference/get-spaces
+        https://developer.twitter.com/en/docs/twitter-api/spaces/lookup/api-reference/get-spaces-by-creator-ids
+        """
+        if ids is not None and user_ids is not None:
+            raise TypeError("Expected IDs or user IDs, not both")
+
+        route = "/2/spaces"
+
+        if ids is not None:
+            params["ids"] = ids
+        elif user_ids is not None:
+            route += "/by/creator_ids"
+            params["user_ids"] = user_ids
+        else:
+            raise TypeError("IDs or user IDs are required")
+
+        return self._make_request(
+            "GET", route, params=params,
+            endpoint_parameters=(
+                "ids", "user_ids", "expansions", "space.fields", "user.fields"
+            ), data_type=Space
+        )
+
+    def get_space(self, id, **params):
+        """get_space(id, *, expansions, space_fields, user_fields)
+
+        Returns a variety of information about a single Space specified by the
+        requested ID.
+
+        Parameters
+        ----------
+        id : Union[List[str], str]
+            Unique identifier of the Space to request.
+        expansions : Union[List[str], str]
+            :ref:`expansions_parameter`
+        space_fields : Union[List[str], str]
+            :ref:`space_fields_parameter`
+        user_fields : Union[List[str], str]
+            :ref:`user_fields_parameter`
+
+        Returns
+        -------
+        Union[dict, requests.Response, Response]
+
+        References
+        ----------
+        https://developer.twitter.com/en/docs/twitter-api/spaces/lookup/api-reference/get-spaces-id
+        """
+        return self._make_request(
+            "GET", f"/2/spaces/{id}", params=params,
+            endpoint_parameters=(
+                "expansions", "space.fields", "user.fields"
+            ), data_type=Space
+        )
+
+    # Batch Compliance
+
+    def get_compliance_jobs(self, type, **params):
+        """get_compliance_jobs(type, *, status)
+
+        Returns a list of recent compliance jobs.
+
+        Parameters
+        ----------
+        type : str
+            Allows to filter by job type - either by tweets or user ID. Only
+            one filter (tweets or users) can be specified per request.
+        status : str
+            Allows to filter by job status. Only one filter can be specified
+            per request.
+            Default: ``all``
+
+        Returns
+        -------
+        Union[dict, requests.Response, Response]
+
+        References
+        ----------
+        https://developer.twitter.com/en/docs/twitter-api/compliance/batch-compliance/api-reference/get-compliance-jobs
+        """
+        params["type"] = type
+        return self._make_request(
+            "GET", "/2/compliance/jobs", params=params,
+            endpoint_parameters=("type", "status")
+        )
+
+    def get_compliance_job(self, id):
+        """Get a single compliance job with the specified ID.
+
+        Parameters
+        ----------
+        id : Union[int, str]
+            The unique identifier for the compliance job you want to retrieve.
+
+        Returns
+        -------
+        Union[dict, requests.Response, Response]
+
+        References
+        ----------
+        https://developer.twitter.com/en/docs/twitter-api/compliance/batch-compliance/api-reference/get-compliance-jobs-id
+        """
+        return self._make_request("GET", f"/2/compliance/jobs/{id}")
+
+    def create_compliance_job(self, type, *, name=None, resumable=None):
+        """Creates a new compliance job for Tweet IDs or user IDs.
+
+        A compliance job will contain an ID and a destination URL. The
+        destination URL represents the location that contains the list of IDs
+        consumed by your app.
+
+        You can run one batch job at a time.
+
+        Parameters
+        ----------
+        type : str
+            Specify whether you will be uploading tweet or user IDs. You can
+            either specify tweets or users.
+        name : str
+            A name for this job, useful to identify multiple jobs using a label
+            you define.
+        resumable : bool
+            Specifies whether to enable the upload URL with support for
+            resumable uploads. If true, this endpoint will return a pre-signed
+            URL with resumable uploads enabled.
+
+        Returns
+        -------
+        Union[dict, requests.Response, Response]
+
+        References
+        ----------
+        https://developer.twitter.com/en/docs/twitter-api/compliance/batch-compliance/api-reference/post-compliance-jobs
+        """
+        json = {"type": type}
+
+        if name is not None:
+            json["name"] = name
+
+        if resumable is not None:
+            json["resumable"] = resumable
+
+        return self._make_request(
+            "POST", "/2/compliance/jobs", json=json
+        )
